import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonComponent, CollaborationIconComponent } from '@ods/system';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { getDataTestIdAttributeOf, getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { CollaborationInVorgangContainerComponent } from './collaboration-in-vorgang-container.component';
import { CollaborationRequestContainerComponent } from './collaboration-request-container/collaboration-request-container.component';

describe('CollaborationInVorgangContainerComponent', () => {
  let component: CollaborationInVorgangContainerComponent;
  let fixture: ComponentFixture<CollaborationInVorgangContainerComponent>;

  const anfrageErstellenButton: string = getDataTestIdAttributeOf('anfrage-erstellen-button');
  const collaborationRequestContainer: string = getDataTestIdOf('collaboration-request-container');

  const service: Mock<CollaborationService> = {
    ...mock(CollaborationService),
    isRequestFormVisible: jest.fn().mockReturnValue(of(false)),
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        CollaborationInVorgangContainerComponent,
        MockComponent(ButtonComponent),
        MockComponent(CollaborationRequestContainerComponent),
        MockComponent(CollaborationIconComponent),
      ],
      providers: [
        {
          provide: CollaborationService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CollaborationInVorgangContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service', () => {
      component.ngOnInit();

      expect(service.isRequestFormVisible).toHaveBeenCalled();
    });
  });

  describe('anfrage erstellen button', () => {
    describe('on request form visibility false', () => {
      beforeEach(() => {
        component.isRequestFormVisible$ = of(false);
      });

      it('should be shown', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, anfrageErstellenButton);
      });

      it('should call service on click', () => {
        fixture.detectChanges();

        dispatchEventFromFixture(fixture, anfrageErstellenButton, 'clickEmitter');

        expect(service.showRequestForm).toHaveBeenCalled();
      });
    });

    it('should be hidden if request form visibility is true', () => {
      component.isRequestFormVisible$ = of(true);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, anfrageErstellenButton);
    });
  });

  describe('zustaendige stelle', () => {
    describe('on request form visibility true', () => {
      beforeEach(() => {
        component.isRequestFormVisible$ = of(true);
      });
      it('should be shown', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, collaborationRequestContainer);
      });

      it('should call service on hideFormular output', () => {
        fixture.detectChanges();

        dispatchEventFromFixture(fixture, collaborationRequestContainer, 'hideRequestForm');

        expect(service.hideRequestForm).toHaveBeenCalled();
      });
    });

    it('should be hidden if request form visibility is false', () => {
      component.isRequestFormVisible$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, collaborationRequestContainer);
    });
  });
});
