package de.ozgcloud.alfa.collaboration;

import static de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.UriTemplate;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class CollaborationVorgangProcessorTest {

	@Spy
	@InjectMocks
	private CollaborationVorgangProcessor processor;

	@Mock
	private CurrentUserService currentUserService;

	private final UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();

	@Nested
	class TestProcess {

		@Nested
		class OnNullVorgang {

			@Test
			void shouldNotAddLinksIfVorgangIsNull() {
				var model = processor.process(new EntityModel<>() {
				});

				assertThat(model.hasLinks()).isFalse();
			}
		}

		@Nested
		class OnNonNullVorgang {

			@BeforeEach
			void prepareBuilder() {
				initUserProfileUrlProvider(urlProvider);
			}

			@Test
			void shouldAddCreateCollaborationRequestRelation() {
				when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(true);

				var model = processor.process(EntityModel.of(VorgangWithEingangTestFactory.create()));

				assertThat(model.getLink(CollaborationVorgangProcessor.REL_CREATE_COLLABORATION_REQUEST)).isPresent().get()
						.extracting(Link::getHref)
						.isEqualTo(UriTemplate.of(CommandController.CommandByRelationController.COMMAND_BY_RELATION_PATH)
								.expand(VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.VERSION).toString());
			}

			@Test
			void shouldNotAddCreateCollaborationRequestRelation() {
				when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(false);

				var model = processor.process(EntityModel.of(VorgangWithEingangTestFactory.create()));

				assertThat(model.getLink(CollaborationVorgangProcessor.REL_CREATE_COLLABORATION_REQUEST)).isEmpty();
			}
		}
	}
}
