package de.ozgcloud.alfa.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mock;

import com.thedeanda.lorem.LoremIpsum;

class CommandTest {

	@Nested
	class TestIsFinishedSuccessfully {

		@Test
		void shouldReturnTrue() {
			var command = CommandTestFactory.createBuilder().status(CommandStatus.FINISHED).build();

			var isDoneSuccessfully = command.isFinishedSuccessfully();

			assertThat(isDoneSuccessfully).isTrue();
		}

		@ParameterizedTest
		@EnumSource(names = "FINISHED", mode = Mode.EXCLUDE)
		void shouldReturnFalseOnStatusNotFinished(CommandStatus commandStatus) {
			var command = CommandTestFactory.createBuilder().status(commandStatus).build();

			var isDoneSuccessfully = command.isFinishedSuccessfully();

			assertThat(isDoneSuccessfully).isFalse();
		}

		@ParameterizedTest
		@EnumSource
		void shouldReturnFalseOnErrorMessage(CommandStatus commandStatus) {
			var command = CommandTestFactory.createBuilder().status(commandStatus).errorMessage(LoremIpsum.getInstance().getWords(1)).build();

			var isDoneSuccessfully = command.isFinishedSuccessfully();

			assertThat(isDoneSuccessfully).isFalse();
		}

	}

	@Nested
	class TestIsNotDone {

		@Mock
		private CommandStatus commandStatus;
		private Command command;

		@BeforeEach
		void setUp() {
			command = CommandTestFactory.createBuilder().status(commandStatus).build();
		}

		@Test
		void shouldCallIsNotDone() {
			command.isNotDone();

			verify(commandStatus).isNotDone();
		}

		@ParameterizedTest
		@ValueSource(booleans = { true, false })
		void shouldReturnIsNotDone(boolean isNotDone) {
			when(commandStatus.isNotDone()).thenReturn(isNotDone);

			var result = command.isNotDone();

			assertThat(result).isEqualTo(isNotDone);
		}

	}

}